require File.expand_path(File.dirname(__FILE__) + '/../result')
require File.expand_path(File.dirname(__FILE__) + '/prevoty/smartfilter')

class SmartFilterBadRuleKey < Exception; end

class SmartFilter < PrevotySmartFilter
  alias_method :api_key, :key
  alias_method :api_key=, :key=

  attr_accessor :rule_key, :input, :output, :response

  def initialize(key = nil)
    @config        = SmartFilter.config(Stoolie.config.smart_filter)

    super(key)

    @rule_key       = key || @config[:rule_key]
    @api_key = @key = @config[:api_key]
  end

  def filter!(input, rule_key)
    rule_key ||= @rule_key
    raise SmartFilterBadRuleKey.new unless rule_key

    super(input, rule_key)
  end

  # Public: allows changing the thresholds at which the xss, spam & offensive methods will trigger
  #
  # Returns a hash
  def self.config(opts =  {})
    opts[:blacklisted_phrases_threshold] ||= 1
    opts[:flagged_phrases_threshold]     ||= 6
    opts[:profanity_threshold]           ||= 3
    opts[:link_density_threshold]        ||= 3
    opts[:spam_features_threshold]       ||= 2
    opts[:javascript_threshold]          ||= 1
    opts[:invalid_tags_threshold]        ||= 1
    opts
  end

  # Public: retrieves the threshold (configured or default) for a given statistic
  #
  # Returns a hash of the form {'statistic' => ${threshold value}}
  def self.stats_to_thresholds
    map = {}
    map['blacklisted_phrases']            = config[:blacklisted_phrases_threshold]
    map['flagged_phrases']                = config[:flagged_phrases_threshold]
    map['prevoty_profanity_features']     = config[:profanity_threshold]
    map['prevoty_link_density']           = config[:link_density_threshold]
    map['prevoty_spam_features']          = config[:spam_features_threshold]

    ['invalid_protocols', 'invalid_attributes', 'invalid_tags'].each do |s|
      map[s] = config[:invalid_tags_threshold]
    end
    ['javascript_tags', 'javascript_protocols', 'javascript_attributes'].each do |s|
      map[s] = config[:javascript_threshold]
    end

    map
  end

  def stat_to_threshold(stat)
    SmartFilter.stats_to_thresholds[stat.to_s]
  end

  # Public: calls the filter method to test the input
  #
  # input - the text to be analyzed
  #
  # Examples
  #
  #   result = analyze('<script>text</script>')
  #   # => #<Stoolie::Result...>
  #   result.insecure?
  #   # => true
  #
  # Returns a Stoolie::Result object
  def analyze(input = '')
    @input    = input
    @response = filter!(@input, rule_key)
    @output   = @response['output']
    @stats    = @response['statistics']

    @result   = Stoolie::Result.new(self)
  end

  # Public: checks the result statistics against the thresholds for security
  #
  # Returns a boolean
  def is_insecure?
    return false unless @stats

    beyond_threshold?(['javascript_tags', 'javascript_protocols', 'javascript_attributes', 'invalid_protocols', 'invalid_attributes'])
  end

  # Public: checks the result statistics against the thresholds for spam
  #
  # Returns a boolean
  def is_spam?
    return false unless @stats

    beyond_threshold?(['prevoty_spam_features']) || link_density(@stats['prevoty_link_density']) >= stat_to_threshold('prevoty_link_density')
  end

  # Public: checks the result statistics against the thresholds for blacklisted phrases
  #
  # Returns a boolean
  def is_blacklisted?
    return false unless @stats

    beyond_threshold?(['blacklisted_phrases'])
  end

  # Public: checks the result statistics against the thresholds for profanity
  #
  # Returns a boolean
  def is_offensive?
    return false unless @stats

    beyond_threshold?(['flagged_phrases', 'prevoty_profanity_features'])
  end

  private

    def link_density(links = {})
      links.values.inject(&:+) || 0
    end

    def beyond_threshold?(stats = [])
      stats.any? { |stat| @stats[stat] >= stat_to_threshold(stat) }
    end

end
